#!/usr/bin/python3

"""
 * kare configures network printers
 *
 * Copyright (C) 2023-2025, KylinSoft Co., Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Xiangwei Liu <liuxiangwei@kylinos.cn>
 *
 """

import subprocess
import time
import threading
from urllib.parse import urlparse
import socket
import re

script_path = '/usr/bin/kare_setup_printer.sh'
lock = threading.Lock()


def get_current_printers():
    """获取当前所有在线打印机URI"""
    try:
        result = subprocess.getoutput(['ippfind'])
        return {line for line in result.splitlines() if line.startswith('ipp://')}
    except Exception as e:
        print(f"获取打印机列表失败: {e}")
        return set()


def extract_printer_name(ipptool_output):
    """从ipptool输出中提取打印机型号"""
    pattern = r'^\s*printer-make-and-model\b[^=]*=\s*"?([^"]+)"?'
    for line in ipptool_output.splitlines():
        match = re.match(pattern, line, re.IGNORECASE)
        if match:
            return bytes(match.group(1).strip(), "utf-8").decode("unicode_escape")


def resolve_ip_address(printer_uri):
    """解析打印机IP地址"""
    parsed = urlparse(printer_uri)
    try:
        return f"socket://{socket.gethostbyname(parsed.hostname)}"
    except socket.gaierror:
        print(f"无法解析主机名: {parsed.hostname}")
        return None


def process_printer(printer_uri):
    """执行打印机驱动匹配（返回是否成功）"""
    try:
        # 获取打印机属性
        result = subprocess.run(
            ["ipptool", "-tv", printer_uri, "get-printer-attributes.test"],
            capture_output=True, text=True, check=True
        )

        # 提取打印机型号
        if not (printer_name := extract_printer_name(result.stdout)):
            print(f"无法识别打印机型号: {printer_uri}")
            return False

        # 解析IP地址
        if not (resolved_uri := resolve_ip_address(printer_uri)):
            return False

        # 执行安装脚本
        subprocess.run(
            [script_path, printer_name, resolved_uri],
            check=True,
            capture_output=True,
            text=True
        )
        print(f"成功处理打印机: {printer_name} ({resolved_uri})")
        return True

    except subprocess.CalledProcessError as e:
        print(f"驱动安装失败: {e.stderr}")
        return False
    except Exception as e:
        print(f"处理异常: {str(e)}")
        return False


def printer_monitor():
    """每秒执行全量检测"""
    while True:
        current_printers = get_current_printers()

        with lock:
            for printer_uri in current_printers:
                process_printer(printer_uri)

        time.sleep(1)


if __name__ == "__main__":
    # 初始化处理所有现有打印机
    with lock:
        initial_printers = get_current_printers()
        for uri in initial_printers:
            process_printer(uri)

    # 启动监控线程
    monitor_thread = threading.Thread(target=printer_monitor)
    monitor_thread.daemon = True
    monitor_thread.start()

    try:
        while True:
            time.sleep(3600)
    except KeyboardInterrupt:
        print("\n监控服务已停止")







# # 用于存储上一次检测到的打印机主机名集合
# previous_printers = set()
#
# # 锁对象，用于线程安全地访问previous_printers
# printers_lock = threading.Lock()
#
# # Shell 脚本的路径
# script_path = '/usr/bin/kare_setup_printer.sh'
#
#
# def get_current_printers():
#     """通过ippfind获取当前打印机的列表，并返回主机名集合"""
#     try:
#         result = subprocess.run(['ippfind'], capture_output=True, text=True, check=True)
#         current_printers = set()
#         for line in result.stdout.splitlines():
#             if line.startswith('ipp://'):
#                 uri = line
#                 # print("===========uri:" + uri)
#                 current_printers.add(uri)
#         return current_printers
#     except Exception as e:
#         # print(f"Error running ippfind: {e}")
#         return set()
#
#
# def extract_printer_name(ipptool_output):
#     pattern = r'^\s*printer-make-and-model\b[^=]*=\s*"?([^"]+)"?'
#
#     for line in ipptool_output.splitlines():
#         # 使用正则表达式进行模式匹配
#         # print("------" + line)
#         match = re.match(pattern, line, re.IGNORECASE)
#         if match:
#             # 提取并清理值
#             value = match.group(1).strip()
#             # 处理可能存在的转义字符
#             return bytes(value, "utf-8").decode("unicode_escape")
#
#
# def replace_host_with_ip(original_uri):
#     parsed = urlparse(original_uri)
#
#     try:
#         # 解析主机名获取IP地址
#         ip_address = socket.gethostbyname(parsed.hostname)
#     except socket.gaierror as e:
#         raise ValueError(f"无法解析主机名: {parsed.hostname}") from e
#
#     return f"socket://{ip_address}"
#
#
# def monitor_printers():
#     global previous_printers
#     """每隔一秒监测一次打印机列表，检测新添加的打印机并获取其名称"""
#     while True:
#         try:
#             with printers_lock:
#                 current_printers = get_current_printers()
#                 new_printers = current_printers - previous_printers
#
#             if not new_printers:
#                 print("No new printers found.")
#
#             for new_printer in new_printers:
#                 try:
#                     print(f"Checking new printer: {new_printer}")
#
#                     # 使用ipptool获取打印机属性
#                     result = subprocess.run(
#                         ["ipptool", "-tv", new_printer, "get-printer-attributes.test"],
#                         capture_output=True, text=True, check=True
#                     )
#                     # print("===========result:" + result.stdout)
#                     # 从输出中提取打印机名称
#                     printer_name = extract_printer_name(result.stdout)
#                     # if printer_name:
#                         # print("===========printer_name:" + printer_name)
#
#                     if printer_name:
#                         try:
#                             new_printer = replace_host_with_ip(new_printer)
#                             # print("===========new_printer:" + str(new_printer))
#                             # socket://10.41.121.131
#                             args = [printer_name, new_printer]
#                             result = subprocess.run([script_path] + args, check=True, capture_output=True, text=True,
#                                                     shell=False)
#
#                             # 输出脚本的标准输出和错误（如果有）
#                             print(f"Standard Output:\n{result.stdout}")
#                             if result.stderr:
#                                 print(f"Standard Error:\n{result.stderr}")
#                         except subprocess.CalledProcessError as e:
#                             print(f"The script exited with error code {e.returncode}")
#                             print(f"Standard Error:\n{e.stderr}")
#                     else:
#                         print(f"Failed to extract printer name for {new_printer}")
#
#                 except subprocess.CalledProcessError as e:
#                     print(f"ipptool failed for {new_printer}: {e}")
#                 except Exception as e:
#                     print(f"An error occurred while processing {new_printer}: {e}")
#
#             with printers_lock:
#                 previous_printers = current_printers
#
#         except Exception as e:
#             print(f"An unexpected error occurred in the monitor loop: {e}")
#
#         time.sleep(1)
#
#
# if __name__ == "__main__":
#     # 首次获取打印机列表以初始化previous_printers
#     with printers_lock:
#         previous_printers = get_current_printers()
#
#     # 启动监测线程
#     monitor_thread = threading.Thread(target=monitor_printers)
#     monitor_thread.daemon = True  # 设置为守护线程
#     monitor_thread.start()
#
#     # 主程序保持运行，以便监测线程可以持续工作
#     try:
#         while True:
#             # 主线程可以执行其他管理任务或保持空闲
#             time.sleep(1)  # 简单地休眠以避免忙等待
#     except (KeyboardInterrupt, SystemExit):
#         print("Monitoring stopped.")
#     finally:
#         # 清理工作（如果有的话）可以在这里进行
#         # 注意：由于monitor_thread是守护线程，它会在主线程退出时自动终止
#         pass
